/*******************************************************************************
 * @file
 * @brief Main application
 * @author Microtronics Engineering GmbH
 * @version 01v000
 *******************************************************************************
 * @section License
 * <b>(C) Copyright 2013-2014 Microtronics Engineering GmbH,
 * http://www.microtronics.at</b>
 ******************************************************************************/
#pragma GCC optimize ("O0")

#include <zephyr.h>
#include <device.h>
#include <drivers/uart.h>
#include <sys/printk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef uint8_t u8;
typedef uint32_t u32;

#define MAIN_UART_BUF_LEN (16)


#define MAIN_DUMMYBUF_LEN (256)
u8 au8Main_DummyBuf[MAIN_DUMMYBUF_LEN] = {0};
u8 au8Main_DummyBufOld[MAIN_DUMMYBUF_LEN] = {0};
struct uart_event sMain_UartRxEv;

static void uart_callback(const struct device *dev,
			  struct uart_event *evt,
			  void *user_data)
{

	struct device *uart = user_data;
	switch (evt->type) 
  {
    case UART_TX_DONE:
    {
      printk("Tx sent %d bytes\r\n", evt->data.tx.len);
      break;
    }
    case UART_TX_ABORTED:
    {
      printk("Tx aborted\r\n");
      break;
    }
    case UART_RX_RDY:
    {
      // 0123456789 <- just to have some text to copy
      u32 u32Idx;
      memcpy(&sMain_UartRxEv, evt, sizeof(sMain_UartRxEv)); /* copy just for the watch window */
      memset(au8Main_DummyBuf, 0, MAIN_DUMMYBUF_LEN);
      memcpy(au8Main_DummyBuf, evt->data.rx.buf, MAIN_UART_BUF_LEN);

      for(u32Idx=0; u32Idx < MAIN_DUMMYBUF_LEN; u32Idx++)
      {
        if(au8Main_DummyBufOld[u32Idx] != au8Main_DummyBuf[u32Idx])
          break;
      }
      printk("evt len = %d, evt offs = %d, actual data in %lu\r\n", evt->data.rx.len, evt->data.rx.offset, u32Idx);
      memcpy(au8Main_DummyBufOld, au8Main_DummyBuf, MAIN_DUMMYBUF_LEN);
      for(u32 i = 0; i < evt->data.rx.len; i++)
      {
        printk("%c", evt->data.rx.buf[evt->data.rx.offset + i]);
      }
      printk("\r\n\n");
      

      break;
    }
    case UART_RX_BUF_REQUEST:
    {
      u8* pu8UartBuf;

      printk("uart rx buf req\r\n");
      pu8UartBuf = k_malloc(MAIN_UART_BUF_LEN);
      uart_rx_buf_rsp(uart, pu8UartBuf, MAIN_UART_BUF_LEN);
      break;
    }
    case UART_RX_BUF_RELEASED:
    {
      printk("freeing buffer\r\n");
      k_free(evt->data.rx_buf.buf);
      break;
    }
    case UART_RX_DISABLED:
    {
      break;
    }
    case UART_RX_STOPPED:
    {
      break;
    }
  }
}

int main(void)
{
  const struct device *uart_dev  = DEVICE_DT_GET(DT_NODELABEL(uart0)); /* "printk" uart */
  const struct device *uartTest_dev = DEVICE_DT_GET(DT_NODELABEL(uart1)); /* "rx" uart */
  struct uart_config uart_cfg;
  struct uart_config uart1_cfg;
  u8* pu8UartBuf;

  uart_config_get(uart_dev, &uart_cfg);
  uart_cfg.baudrate = 115200;
  uart_configure(uart_dev, &uart_cfg); 

  uart_config_get(uartTest_dev, &uart1_cfg);
  uart1_cfg.baudrate = 115200;
  uart_configure(uartTest_dev, &uart1_cfg);  
  printk("MAIN START!\r\n");

  uart_callback_set(uartTest_dev, uart_callback, (void*)uartTest_dev);
  pu8UartBuf = k_malloc(MAIN_UART_BUF_LEN);
  uart_rx_enable(uartTest_dev, pu8UartBuf, MAIN_UART_BUF_LEN, 10);

  memset(au8Main_DummyBuf, 0, MAIN_DUMMYBUF_LEN);
  while(1)
  {
  }
}
